import ORG.opengroup.axtp.*;
import ORG.opengroup.axtp.tp.*;
import ORG.opengroup.axtp.transaction.*;

public class Server implements ExecutionContext
{
	private final static int INIT=0;
	private final static int OK=1;
	private final static int ERR=2;
	private final static int ZOK=3;
	private final static int ZERR=4;
	private final static int FIN=5;

	private long xid=0;
	private TransactionServer ts;
	private int state=INIT;
	private RetryDelay retryDelay=null;
	private ExecutionObject eo=null;
	private long lastTime=0;

	public void setXID(long xid)
	{
		this.xid=xid;
	}

	public long getXID()
	{
		return xid;
	}

	public void setTransactionServer(TransactionServer ts)
	{
		this.ts=ts;
	}

	public void apply(TransactionObject to)
	{
		switch(state)
		{
			case INIT:
			{
				if (to instanceof X)
				{
					retryDelay=ts.cloneRetryDelay();
					retryDelay.resetDelay();
					state=OK;
					long delay=retryDelay.getCurrentDelay();
					ts.setTimer(this,delay);
					eo=ts.getTransactionFactory().createA();
					eo.setXID(xid);
					lastTime=System.currentTimeMillis();
					ts.sendToClient(eo);
				}
				return;
			}
			case OK:
			{
				if (to instanceof X)
				{
					long tm=System.currentTimeMillis();
					if ((tm-lastTime)>(retryDelay.getInitialDelay()/2))
					{
						retryDelay.resetDelay();
						long delay=retryDelay.getCurrentDelay();
						ts.setTimer(this,delay);
						lastTime=tm;
						ts.sendToClient(eo);
					}
				}
				else if (to instanceof Z)
				{
					state=ZOK;
					ts.clearTimer(this);
					retryDelay=null;
					eo=null;
				}
				else if (to instanceof R)
				{
					state=FIN;
					ts.clearTimer(this);
					retryDelay=null;
					eo=null;
				}
				else if (to instanceof C)
				{
					state=FIN;
					ts.clearTimer(this);
					retryDelay=null;
					eo=null;
				}
				return;
			}
			case ERR:
			{
				if (to instanceof X)
				{
					long tm=System.currentTimeMillis();
					if ((tm-lastTime)>(retryDelay.getInitialDelay()/2))
					{
						retryDelay.resetDelay();
						long delay=retryDelay.getCurrentDelay();
						ts.setTimer(this,delay);
						lastTime=tm;
						ts.sendToClient(eo);
					}
				}
				else if (to instanceof Z)
				{
					state=ZERR;
					ts.clearTimer(this);
					retryDelay=null;
					eo=null;
				}
				else if (to instanceof R)
				{
					state=FIN;
					ts.clearTimer(this);
					retryDelay=null;
					eo=null;
				}
				else if (to instanceof C)
				{
					state=FIN;
					ts.clearTimer(this);
					retryDelay=null;
					eo=null;
				}
				return;
			}
			case ZOK:
			{
				if (to instanceof R)
				{
					state=FIN;
				}
				else if (to instanceof C)
				{
					state=FIN;
				}
				return;
			}
			case ZERR:
			{
				if (to instanceof R)
				{
					state=FIN;
				}
				else if (to instanceof C)
				{
					state=FIN;
				}
				return;
			}
		}
	}

	public void timeout()
	{
		if (ts.isNoisy())
			System.out.println("Server.timeout XID="+xid+" state="+state);
		switch(state)
		{
			case OK:
			case ERR:
			{
				retryDelay.incrimentDelay();
				long delay=retryDelay.getCurrentDelay();
				ts.setTimer(this,delay);
				lastTime=System.currentTimeMillis();
				ts.sendToClient(eo);
				return;
			}
		}
	}

	public void z()
	{
		switch(state)
		{
			case OK:
			{
				state=ZOK;
				ts.clearTimer(this);
				retryDelay=null;
				eo=null;
				return;
			}
			case ERR:
			{
				state=ZERR;
				ts.clearTimer(this);
				retryDelay=null;
				eo=null;
				return;
			}
		}
	}
}
