//Copyright (c) 1997 The Open Group
//
//Permission to use, copy, modify, and distribute this software and its documentation for
//any purpose and without fee is hereby granted, provided that the above copyright notice
//appears in all copies and that both the copyright notice and this permission notice
//appear in supporting documentation, and that the name of ("TOG") or The Open Group not
//be used in advertising or publicity pertaining to distribution of the software without
//specific, written prior permission.
//
//THE OPEN GROUP DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL
//IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT
//SHALL TOG BE LIABLE FOR ANY SPECIAL, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
//WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN ACTION OF CONTRACT,
//NEGLIGENCE, OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
//PERFORMANCE OF THIS SOFTWARE. 


package ORG.opengroup.misc;

import java.util.*;

/**
 * This class contains two hashtables:
 * ht - contains key Object mapped to a Vector of values
 * inverseHt - is the inverse of ht, in the sense that ht 
 * values are the keys of inverseHt and ht keys become the values. 
 * But since ht contains a Vector of values the two hashtables are
 * not symmetric.
 */

public class TwinAsymmetricHashtable
{
  Hashtable ht = null;
  Hashtable inverseHt = null;

	/**
	 * Creates a new instance
	 */
  public TwinAsymmetricHashtable(){
    ht = new Hashtable();
    inverseHt = new Hashtable();
  }

	/**
	 * Adds a new (key, value) pair
	 */
  public synchronized void put(Object key, Object value){
    Vector valueList = null;
    Object list = ht.get(key);
    if (list == null){
      valueList = new Vector();
    }
    else {
      valueList = (Vector)list;
    }
    valueList.addElement(value);
    ht.put(key,valueList);
    inverseHt.put(value,key);
  }

	/**
	 * returns a new (key, value) pair
	 */
  public synchronized Object get(Object key){
    return ht.get(key);
  }

	/**
	 * Returns the inverse (key, value) pair
	 */
  public synchronized Object getInverse(Object key){
    return inverseHt.get(key);
  }

  /**
   * Removes the key (and corresponding values) from ht
   * Removes each value (key for inverseHt) from the inverseHt
   */
  public synchronized Object remove(Object key){
    Object list = ht.remove(key);
    if (list != null) {
      Vector valueList = (Vector)list;
      for (int i = 0;i < valueList.size();i++)
	inverseHt.remove(valueList.elementAt(i));
    }    
    return list;
  }

	/**
	 * Removes the inverse (key,value) pair
	 */
  public synchronized Object removeInv(Object key){
    Object value = inverseHt.remove(key);
    if (value != null) {
      Object list = ht.get(value);
      Vector valueList = (Vector)list;
      valueList.removeElement(key);
      if (valueList.isEmpty())
	ht.remove(value);
    }
    return value;
  }

} // DoubleHashtable


